@@ -26,9 +26,8 @@ import gi
 gi.require_version('Gedit', '3.0')
 gi.require_version('Gtk', '3.0')
 gi.require_version('Vte', '2.91')
-from gi.repository import GObject, GLib, Gio, Pango, Gdk, Gtk, Gedit, Vte
-
-from gpdefs import *
+gi.require_version('Tepl', '6')
+from gi.repository import GObject, GLib, Gio, Pango, Gdk, Gtk, Gedit, Tepl, Vte
 
 try:
     import gettext
@@ -44,6 +43,22 @@ class GeditTerminal(Vte.Terminal):
         'audible_bell'          : False,
     }
 
+    SETTINGS_SCHEMA_ID_BASE = "org.gnome.Terminal.ProfilesList"
+    SETTINGS_SCHEMA_ID_FALLBACK = "org.gnome.gedit.plugins.terminal"
+    SETTING_KEY_PROFILE_USE_SYSTEM_FONT = "use-system-font"
+    SETTING_KEY_PROFILE_FONT = "font"
+    SETTING_KEY_PROFILE_USE_THEME_COLORS = "use-theme-colors"
+    SETTING_KEY_PROFILE_FOREGROUND_COLOR = "foreground-color"
+    SETTING_KEY_PROFILE_BACKGROUND_COLOR = "background-color"
+    SETTING_KEY_PROFILE_PALETTE = "palette"
+    SETTING_KEY_PROFILE_CURSOR_BLINK_MODE = "cursor-blink-mode"
+    SETTING_KEY_PROFILE_CURSOR_SHAPE = "cursor-shape"
+    SETTING_KEY_PROFILE_AUDIBLE_BELL = "audible-bell"
+    SETTING_KEY_PROFILE_SCROLL_ON_KEYSTROKE = "scroll-on-keystroke"
+    SETTING_KEY_PROFILE_SCROLL_ON_OUTPUT = "scroll-on-output"
+    SETTING_KEY_PROFILE_SCROLLBACK_UNLIMITED = "scrollback-unlimited"
+    SETTING_KEY_PROFILE_SCROLLBACK_LINES = "scrollback-lines"
+
     TARGET_URI_LIST = 200
 
     def __init__(self):
@@ -70,39 +85,50 @@ class GeditTerminal(Vte.Terminal):
 
     def do_drag_data_received(self, drag_context, x, y, data, info, time):
         if info == self.TARGET_URI_LIST:
-            self.feed_child(' '.join(["'" + Gio.file_new_for_uri(item).get_path() + "'" for item in Gedit.utils_drop_get_uris(data)]), -1)
+            self.feed_child(' '.join(["'" + Gio.file_new_for_uri(item).get_path() + "'" for item in Gedit.utils_drop_get_uris(data)]).encode('utf-8'))
             Gtk.drag_finish(drag_context, True, False, time);
         else:
             Vte.Terminal.do_drag_data_received(self, drag_context, x, y, data, info, time)
 
-    def settings_try_new(self, schema):
-        schemas = Gio.Settings.list_schemas()
-        if not schemas:
-            return None
-
-        for s in schemas:
-            if s == schema:
-                return Gio.Settings.new(schema)
-
-        return None
-
     def get_profile_settings(self):
-        profiles = self.settings_try_new("org.gnome.Terminal.ProfilesList")
+        fallback_settings = Gio.Settings.new(self.SETTINGS_SCHEMA_ID_FALLBACK)
 
-        if profiles:
-            default_path = "/org/gnome/terminal/legacy/profiles:/:" + profiles.get_string("default") + "/"
-            settings = Gio.Settings.new_with_path("org.gnome.Terminal.Legacy.Profile",
-                                                  default_path)
-        else:
-            settings = Gio.Settings.new("org.gnome.gedit.plugins.terminal")
+        if not Tepl.utils_can_use_gsettings_schema(self.SETTINGS_SCHEMA_ID_BASE):
+            return fallback_settings
 
-        return settings
+        profiles = Gio.Settings.new(self.SETTINGS_SCHEMA_ID_BASE)
+        if not Tepl.utils_can_use_gsettings_key(profiles, "default"):
+            return fallback_settings
+
+        default_path = "/org/gnome/terminal/legacy/profiles:/:" + profiles.get_string("default") + "/"
+
+        if not Tepl.utils_can_use_gsettings_schema("org.gnome.Terminal.Legacy.Profile"):
+            return fallback_settings
+
+        settings = Gio.Settings.new_with_path("org.gnome.Terminal.Legacy.Profile", default_path)
+
+        if (Tepl.utils_can_use_gsettings_key(settings, self.SETTING_KEY_PROFILE_USE_SYSTEM_FONT) and
+            Tepl.utils_can_use_gsettings_key(settings, self.SETTING_KEY_PROFILE_FONT) and
+            Tepl.utils_can_use_gsettings_key(settings, self.SETTING_KEY_PROFILE_USE_THEME_COLORS) and
+            Tepl.utils_can_use_gsettings_key(settings, self.SETTING_KEY_PROFILE_FOREGROUND_COLOR) and
+            Tepl.utils_can_use_gsettings_key(settings, self.SETTING_KEY_PROFILE_BACKGROUND_COLOR) and
+            Tepl.utils_can_use_gsettings_key(settings, self.SETTING_KEY_PROFILE_PALETTE) and
+            Tepl.utils_can_use_gsettings_key(settings, self.SETTING_KEY_PROFILE_CURSOR_BLINK_MODE) and
+            Tepl.utils_can_use_gsettings_key(settings, self.SETTING_KEY_PROFILE_CURSOR_SHAPE) and
+            Tepl.utils_can_use_gsettings_key(settings, self.SETTING_KEY_PROFILE_AUDIBLE_BELL) and
+            Tepl.utils_can_use_gsettings_key(settings, self.SETTING_KEY_PROFILE_SCROLL_ON_KEYSTROKE) and
+            Tepl.utils_can_use_gsettings_key(settings, self.SETTING_KEY_PROFILE_SCROLL_ON_OUTPUT) and
+            Tepl.utils_can_use_gsettings_key(settings, self.SETTING_KEY_PROFILE_SCROLLBACK_UNLIMITED) and
+            Tepl.utils_can_use_gsettings_key(settings, self.SETTING_KEY_PROFILE_SCROLLBACK_LINES)):
+            return settings
+
+        return fallback_settings
 
     def get_font(self):
-        if self.profile_settings.get_boolean("use-system-font"):
+        if self.profile_settings.get_boolean(self.SETTING_KEY_PROFILE_USE_SYSTEM_FONT):
             font = self.system_settings.get_string("monospace-font-name")
         else:
-            font = self.profile_settings.get_string("font")
+            font = self.profile_settings.get_string(self.SETTING_KEY_PROFILE_FONT)
 
         return font
 
@@ -122,16 +148,16 @@ class GeditTerminal(Vte.Terminal):
         bg = context.get_background_color(Gtk.StateFlags.NORMAL)
         palette = []
 
-        if not self.profile_settings.get_boolean("use-theme-colors"):
-            fg_color = self.profile_settings.get_string("foreground-color")
+        if not self.profile_settings.get_boolean(self.SETTING_KEY_PROFILE_USE_THEME_COLORS):
+            fg_color = self.profile_settings.get_string(self.SETTING_KEY_PROFILE_FOREGROUND_COLOR)
             if fg_color != "":
                 fg = Gdk.RGBA()
                 parsed = fg.parse(fg_color)
-            bg_color = self.profile_settings.get_string("background-color")
+            bg_color = self.profile_settings.get_string(self.SETTING_KEY_PROFILE_BACKGROUND_COLOR)
             if bg_color != "":
                 bg = Gdk.RGBA()
                 parsed = bg.parse(bg_color)
-        str_colors = self.profile_settings.get_strv("palette")
+        str_colors = self.profile_settings.get_strv(self.SETTING_KEY_PROFILE_PALETTE)
         if str_colors:
             for str_color in str_colors:
                 try:
@@ -143,18 +169,17 @@ class GeditTerminal(Vte.Terminal):
                     break
 
         self.set_colors(fg, bg, palette)
-        self.set_cursor_blink_mode(self.profile_settings.get_enum("cursor-blink-mode"))
-        self.set_cursor_shape(self.profile_settings.get_enum("cursor-shape"))
-        self.set_audible_bell(self.profile_settings.get_boolean("audible-bell"))
-        self.set_allow_bold(self.profile_settings.get_boolean("allow-bold"))
-        self.set_scroll_on_keystroke(self.profile_settings.get_boolean("scroll-on-keystroke"))
-        self.set_scroll_on_output(self.profile_settings.get_boolean("scroll-on-output"))
+        self.set_cursor_blink_mode(self.profile_settings.get_enum(self.SETTING_KEY_PROFILE_CURSOR_BLINK_MODE))
+        self.set_cursor_shape(self.profile_settings.get_enum(self.SETTING_KEY_PROFILE_CURSOR_SHAPE))
+        self.set_audible_bell(self.profile_settings.get_boolean(self.SETTING_KEY_PROFILE_AUDIBLE_BELL))
+        self.set_scroll_on_keystroke(self.profile_settings.get_boolean(self.SETTING_KEY_PROFILE_SCROLL_ON_KEYSTROKE))
+        self.set_scroll_on_output(self.profile_settings.get_boolean(self.SETTING_KEY_PROFILE_SCROLL_ON_OUTPUT))
         self.set_audible_bell(self.defaults['audible_bell'])
 
-        if self.profile_settings.get_boolean("scrollback-unlimited"):
+        if self.profile_settings.get_boolean(self.SETTING_KEY_PROFILE_SCROLLBACK_UNLIMITED):
             lines = -1
         else:
-            lines = self.profile_settings.get_int("scrollback-lines")
+            lines = self.profile_settings.get_int(self.SETTING_KEY_PROFILE_SCROLLBACK_LINES)
         self.set_scrollback_lines(lines)
 
     def on_profile_settings_changed(self, settings, key):
@@ -278,12 +303,12 @@ class GeditTerminalPanel(Gtk.Box):
         menu.attach_to_widget(self, None)
 
         if event is not None:
-            menu.popup(None, None, None, None, event.button, event.time)
+            menu.popup_at_pointer(event)
         else:
-            menu.popup(None, None,
-                       lambda m: Gedit.utils_menu_position_under_widget(m, self),
-                       None,
-                       0, Gtk.get_current_event_time())
+            menu.popup_at_widget(self,
+                                 Gdk.Gravity.NORTH_WEST,
+                                 Gdk.Gravity.SOUTH_WEST,
+                                 None)
             menu.select_first(False)
 
     def copy_clipboard(self):
@@ -296,7 +321,7 @@ class GeditTerminalPanel(Gtk.Box):
 
     def change_directory(self, path):
         path = path.replace('\\', '\\\\').replace('"', '\\"')
-        self._vte.feed_child('cd "%s"\n' % path, -1)
+        self._vte.feed_child(('cd "%s"\n' % path).encode('utf-8'))
         self._vte.grab_focus()
 
 class TerminalPlugin(GObject.Object, Gedit.WindowActivatable):
